@@ -130,91 +130,91 @@ class OpencorporaTag(object):
     # -------------------------------------------------------------
 
     PARTS_OF_SPEECH = frozenset([
-        'NOUN', # имя существительное
-        'ADJF', # имя прилагательное (полное)
-        'ADJS', # имя прилагательное (краткое)
-        'COMP', # компаратив
-        'VERB', # глагол (личная форма)
-        'INFN', # глагол (инфинитив)
-        'PRTF', # причастие (полное)
-        'PRTS', # причастие (краткое)
-        'GRND', # деепричастие
-        'NUMR', # числительное
-        'ADVB', # наречие
-        'NPRO', # местоимение-существительное
-        'PRED', # предикатив
-        'PREP', # предлог
-        'CONJ', # союз
-        'PRCL', # частица
-        'INTJ', # междометие
+        'NOUN',  # имя существительное
+        'ADJF',  # имя прилагательное (полное)
+        'ADJS',  # имя прилагательное (краткое)
+        'COMP',  # компаратив
+        'VERB',  # глагол (личная форма)
+        'INFN',  # глагол (инфинитив)
+        'PRTF',  # причастие (полное)
+        'PRTS',  # причастие (краткое)
+        'GRND',  # деепричастие
+        'NUMR',  # числительное
+        'ADVB',  # наречие
+        'NPRO',  # местоимение-существительное
+        'PRED',  # предикатив
+        'PREP',  # предлог
+        'CONJ',  # союз
+        'PRCL',  # частица
+        'INTJ',  # междометие
     ])
 
     ANIMACY = frozenset([
-        'anim', # одушевлённое
-        'inan', # неодушевлённое
+        'anim',  # одушевлённое
+        'inan',  # неодушевлённое
     ])
 
     GENDERS = frozenset([
-        'masc', # мужской род
-        'femn', # женский род
-        'neut', # средний род
+        'masc',  # мужской род
+        'femn',  # женский род
+        'neut',  # средний род
     ])
 
     NUMBERS = frozenset([
-        'sing', # единственное число
-        'plur', # множественное число
+        'sing',  # единственное число
+        'plur',  # множественное число
     ])
 
     CASES = frozenset([
-        'nomn', # именительный падеж
-        'gent', # родительный падеж
-        'datv', # дательный падеж
-        'accs', # винительный падеж
-        'ablt', # творительный падеж
-        'loct', # предложный падеж
-        'voct', # звательный падеж
-        'gen1', # первый родительный падеж
-        'gen2', # второй родительный (частичный) падеж
-        'acc2', # второй винительный падеж
-        'loc1', # первый предложный падеж
-        'loc2', # второй предложный (местный) падеж
+        'nomn',  # именительный падеж
+        'gent',  # родительный падеж
+        'datv',  # дательный падеж
+        'accs',  # винительный падеж
+        'ablt',  # творительный падеж
+        'loct',  # предложный падеж
+        'voct',  # звательный падеж
+        'gen1',  # первый родительный падеж
+        'gen2',  # второй родительный (частичный) падеж
+        'acc2',  # второй винительный падеж
+        'loc1',  # первый предложный падеж
+        'loc2',  # второй предложный (местный) падеж
     ])
 
     ASPECTS = frozenset([
-        'perf', # совершенный вид
-        'impf', # несовершенный вид
+        'perf',  # совершенный вид
+        'impf',  # несовершенный вид
     ])
 
     TRANSITIVITY = frozenset([
-        'tran', # переходный
-        'intr', # непереходный
+        'tran',  # переходный
+        'intr',  # непереходный
     ])
 
     PERSONS = frozenset([
-        '1per', # 1 лицо
-        '2per', # 2 лицо
-        '3per', # 3 лицо
+        '1per',  # 1 лицо
+        '2per',  # 2 лицо
+        '3per',  # 3 лицо
     ])
 
     TENSES = frozenset([
-        'pres', # настоящее время
-        'past', # прошедшее время
-        'futr', # будущее время
+        'pres',  # настоящее время
+        'past',  # прошедшее время
+        'futr',  # будущее время
     ])
 
     MOODS = frozenset([
-        'indc', # изъявительное наклонение
-        'impr', # повелительное наклонение
+        'indc',  # изъявительное наклонение
+        'impr',  # повелительное наклонение
     ])
 
     VOICES = frozenset([
-        'actv', # действительный залог
-        'pssv', # страдательный залог
+        'actv',  # действительный залог
+        'pssv',  # страдательный залог
     ])
 
     INVOLVEMENT = frozenset([
-        'incl', # говорящий включён в действие
-        'excl', # говорящий не включён в действие
+        'incl',  # говорящий включён в действие
+        'excl',  # говорящий не включён в действие
     ])
 
     # Set this to False (as a class attribute) to disable strict
@@ -231,13 +231,13 @@ class OpencorporaTag(object):
     # Helper attributes for inflection/declension routines
     # ----------------------------------------------------
     _NON_PRODUCTIVE_GRAMMEMES = set(['NUMR', 'NPRO', 'PRED', 'PREP',
-                                   'CONJ', 'PRCL', 'INTJ', 'Apro'])
-    _EXTRA_INCOMPATIBLE = { # XXX: is it a good idea to have these rules?
+                                     'CONJ', 'PRCL', 'INTJ', 'Apro'])
+    _EXTRA_INCOMPATIBLE = {  # XXX: is it a good idea to have these rules?
         'plur': set(['GNdr']),
         # XXX: how to use rules from OpenCorpora?
         # (they have "lexeme/form" separation)
     }
-    _GRAMMEME_INDICES = collections.defaultdict(lambda: 0)
+    _GRAMMEME_INDICES = collections.defaultdict(int)
     _GRAMMEME_INCOMPATIBLE = collections.defaultdict(set)
     KNOWN_GRAMMEMES = set()
 
@@ -256,6 +256,7 @@ class OpencorporaTag(object):
         grammemes_tuple = tuple([intern(str(g)) for g in grammemes])
 
         self._assert_grammemes_are_known(set(grammemes_tuple))
+
         self._grammemes_tuple = grammemes_tuple
         self._POS = self._grammemes_tuple[0]
         self._grammemes_cache = None
@@ -322,6 +323,12 @@ class OpencorporaTag(object):
     def __hash__(self):
         return hash(self._grammemes_tuple)
 
+    def __len__(self):
+        return len(self._grammemes_tuple)
+
+    def __reduce__(self):
+        return self.__class__, (self._str,), None
+
 
     def is_productive(self):
         return not self.grammemes & self._NON_PRODUCTIVE_GRAMMEMES
@@ -331,18 +338,23 @@ class OpencorporaTag(object):
 
     @classmethod
     def grammeme_is_known(cls, grammeme):
-        if not cls.KNOWN_GRAMMEMES:
-            msg = "The class was not properly initialized."
-            raise RuntimeError(msg)
+        cls._assert_grammemes_initialized()
         return grammeme in cls.KNOWN_GRAMMEMES
 
     @classmethod
     def _assert_grammemes_are_known(cls, grammemes):
         if not grammemes <= cls.KNOWN_GRAMMEMES:
+            cls._assert_grammemes_initialized()
             unknown = grammemes - cls.KNOWN_GRAMMEMES
             unknown_repr = ", ".join(["'%s'" % g for g in sorted(unknown)])
             raise ValueError("Grammemes are unknown: {%s}" % unknown_repr)
 
+    @classmethod
+    def _assert_grammemes_initialized(cls):
+        if not cls.KNOWN_GRAMMEMES:
+            msg = "The class was not properly initialized."
+            raise RuntimeError(msg)
+
     def updated_grammemes(self, required):
         """
         Return a new set of grammemes with ``required`` grammemes added
@@ -439,7 +451,7 @@ class CyrillicOpencorporaTag(OpencorporaTag):
         cls._init_alias_map(dict_grammemes)
         super(CyrillicOpencorporaTag, cls)._init_grammemes(dict_grammemes)
 
-        GRAMMEME_INDICES = collections.defaultdict(lambda: 0)
+        GRAMMEME_INDICES = collections.defaultdict(int)
         for name, idx in cls._GRAMMEME_INDICES.items():
             GRAMMEME_INDICES[cls._from_internal_grammeme(name)] = idx
         cls._GRAMMEME_INDICES = GRAMMEME_INDICES
@@ -464,4 +476,4 @@ class CyrillicOpencorporaTag(OpencorporaTag):
 registry = dict()
 
 for tag_type in [CyrillicOpencorporaTag, OpencorporaTag]:
-    registry[tag_type.FORMAT] = tag_type
\ No newline at end of file
+    registry[tag_type.FORMAT] = tag_type
